@@ -1,16 +1,36 @@
 """This module should not be used directly as its API is subject to change. Instead,
 use the `gr.Blocks.load()` or `gr.Interface.load()` functions."""
 
+from __future__ import annotations
+
 import base64
 import json
+import math
+import numbers
+import operator
 import re
+import uuid
+import warnings
 from copy import deepcopy
-from typing import Callable, Dict
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Tuple
 
 import requests
+import websockets
+import yaml
+from packaging import version
 
 import gradio
-from gradio import components, utils
+from gradio import components, exceptions, utils
+from gradio.processing_utils import to_binary
+
+if TYPE_CHECKING:
+    from gradio.components import DataframeData
+
+
+class TooManyRequestsError(Exception):
+    """Raised when the Hugging Face API returns a 429 status code."""
+
+    pass
 
 
 def load_blocks_from_repo(name, src=None, api_key=None, alias=None, **kwargs):
@@ -35,6 +55,58 @@ def load_blocks_from_repo(name, src=None, api_key=None, alias=None, **kwargs):
     return blocks
 
 
+def get_tabular_examples(model_name) -> Dict[str, List[float]]:
+    readme = requests.get(f"https://huggingface.co/{model_name}/resolve/main/README.md")
+    if readme.status_code != 200:
+        warnings.warn(f"Cannot load examples from README for {model_name}", UserWarning)
+        example_data = {}
+    else:
+        yaml_regex = re.search(
+            "(?:^|[\r\n])---[\n\r]+([\\S\\s]*?)[\n\r]+---([\n\r]|$)", readme.text
+        )
+        example_yaml = next(yaml.safe_load_all(readme.text[: yaml_regex.span()[-1]]))
+        example_data = example_yaml.get("widget", {}).get("structuredData", {})
+    if not example_data:
+        raise ValueError(
+            f"No example data found in README.md of {model_name} - Cannot build gradio demo. "
+            "See the README.md here: https://huggingface.co/scikit-learn/tabular-playground/blob/main/README.md "
+            "for a reference on how to provide example data to your model."
+        )
+    # replace nan with string NaN for inference API
+    for data in example_data.values():
+        for i, val in enumerate(data):
+            if isinstance(val, numbers.Number) and math.isnan(val):
+                data[i] = "NaN"
+    return example_data
+
+
+def cols_to_rows(
+    example_data: Dict[str, List[float]]
+) -> Tuple[List[str], List[List[float]]]:
+    headers = list(example_data.keys())
+    n_rows = max(len(example_data[header] or []) for header in headers)
+    data = []
+    for row_index in range(n_rows):
+        row_data = []
+        for header in headers:
+            col = example_data[header] or []
+            if row_index >= len(col):
+                row_data.append("NaN")
+            else:
+                row_data.append(col[row_index])
+        data.append(row_data)
+    return headers, data
+
+
+def rows_to_cols(
+    incoming_data: DataframeData,
+) -> Dict[str, Dict[str, Dict[str, List[str]]]]:
+    data_column_wise = {}
+    for i, header in enumerate(incoming_data["headers"]):
+        data_column_wise[header] = [str(row[i]) for row in incoming_data["data"]]
+    return {"inputs": {"data": data_column_wise}}
+
+
 def get_models_interface(model_name, api_key, alias, **kwargs):
     model_url = "https://huggingface.co/{}".format(model_name)
     api_url = "https://api-inference.huggingface.co/models/{}".format(model_name)
@@ -50,6 +122,15 @@ def get_models_interface(model_name, api_key, alias, **kwargs):
     assert response.status_code == 200, "Invalid model name or src"
     p = response.json().get("pipeline_tag")
 
+    def postprocess_label(scores):
+        sorted_pred = sorted(scores.items(), key=operator.itemgetter(1), reverse=True)
+        return {
+            "label": sorted_pred[0][0],
+            "confidences": [
+                {"label": pred[0], "confidence": pred[1]} for pred in sorted_pred
+            ],
+        }
+
     def encode_to_base64(r: requests.Response) -> str:
         # Handles the different ways HF API returns the prediction
         base64_repr = base64.b64encode(r.content).decode("utf-8")
@@ -76,36 +157,30 @@ def get_models_interface(model_name, api_key, alias, **kwargs):
 
     pipelines = {
         "audio-classification": {
-            # example model: https://hf.co/ehcalabres/wav2vec2-lg-xlsr-en-speech-emotion-recognition
+            # example model: ehcalabres/wav2vec2-lg-xlsr-en-speech-emotion-recognition
             "inputs": components.Audio(source="upload", type="filepath", label="Input"),
             "outputs": components.Label(label="Class"),
-            "preprocess": lambda i: base64.b64decode(
-                i["data"].split(",")[1]
-            ),  # convert the base64 representation to binary
-            "postprocess": lambda r: {
-                i["label"].split(", ")[0]: i["score"] for i in r.json()
-            },
+            "preprocess": lambda i: to_binary,
+            "postprocess": lambda r: postprocess_label(
+                {i["label"].split(", ")[0]: i["score"] for i in r.json()}
+            ),
         },
         "audio-to-audio": {
-            # example model: https://hf.co/speechbrain/mtl-mimic-voicebank
+            # example model: speechbrain/mtl-mimic-voicebank
             "inputs": components.Audio(source="upload", type="filepath", label="Input"),
             "outputs": components.Audio(label="Output"),
-            "preprocess": lambda i: base64.b64decode(
-                i["data"].split(",")[1]
-            ),  # convert the base64 representation to binary
+            "preprocess": to_binary,
             "postprocess": encode_to_base64,
         },
         "automatic-speech-recognition": {
-            # example model: https://hf.co/jonatasgrosman/wav2vec2-large-xlsr-53-english
+            # example model: jonatasgrosman/wav2vec2-large-xlsr-53-english
             "inputs": components.Audio(source="upload", type="filepath", label="Input"),
             "outputs": components.Textbox(label="Output"),
-            "preprocess": lambda i: base64.b64decode(
-                i["data"].split(",")[1]
-            ),  # convert the base64 representation to binary
+            "preprocess": to_binary,
             "postprocess": lambda r: r.json()["text"],
         },
         "feature-extraction": {
-            # example model: hf.co/julien-c/distilbert-feature-extraction
+            # example model: julien-c/distilbert-feature-extraction
             "inputs": components.Textbox(label="Input"),
             "outputs": components.Dataframe(label="Output"),
             "preprocess": lambda x: {"inputs": x},
@@ -115,29 +190,21 @@ def get_models_interface(model_name, api_key, alias, **kwargs):
             "inputs": components.Textbox(label="Input"),
             "outputs": components.Label(label="Classification"),
             "preprocess": lambda x: {"inputs": x},
-            "postprocess": lambda r: {i["token_str"]: i["score"] for i in r.json()},
+            "postprocess": lambda r: postprocess_label(
+                {i["token_str"]: i["score"] for i in r.json()}
+            ),
         },
         "image-classification": {
-            # Example: https://huggingface.co/google/vit-base-patch16-224
+            # Example: google/vit-base-patch16-224
             "inputs": components.Image(type="filepath", label="Input Image"),
             "outputs": components.Label(label="Classification"),
-            "preprocess": lambda i: base64.b64decode(
-                i.split(",")[1]
-            ),  # convert the base64 representation to binary
-            "postprocess": lambda r: {
-                i["label"].split(", ")[0]: i["score"] for i in r.json()
-            },
+            "preprocess": to_binary,
+            "postprocess": lambda r: postprocess_label(
+                {i["label"].split(", ")[0]: i["score"] for i in r.json()}
+            ),
         },
-        # TODO: support image segmentation pipeline -- should we add a new output component type?
-        # 'image-segmentation': {
-        #     # Example: https://hf.co/facebook/detr-resnet-50-panoptic
-        #     'inputs': inputs.Image(label="Input Image", type="filepath"),
-        #     'outputs': outputs.Image(label="Segmentation"),
-        #     'preprocess': lambda i: base64.b64decode(i.split(",")[1]),  # convert the base64 representation to binary
-        #     'postprocess': lambda x: base64.b64encode(x.json()[0]["mask"]).decode('utf-8'),
-        # },
-        # TODO: also: support NER pipeline, object detection, table question answering
         "question-answering": {
+            # Example: deepset/xlm-roberta-base-squad2
             "inputs": [
                 components.Textbox(lines=7, label="Context"),
                 components.Textbox(label="Question"),
@@ -147,29 +214,33 @@ def get_models_interface(model_name, api_key, alias, **kwargs):
                 components.Label(label="Score"),
             ],
             "preprocess": lambda c, q: {"inputs": {"context": c, "question": q}},
-            "postprocess": lambda r: (r.json()["answer"], r.json()["score"]),
+            "postprocess": lambda r: (r.json()["answer"], {"label": r.json()["score"]}),
         },
         "summarization": {
+            # Example: facebook/bart-large-cnn
             "inputs": components.Textbox(label="Input"),
             "outputs": components.Textbox(label="Summary"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: r.json()[0]["summary_text"],
         },
         "text-classification": {
+            # Example: distilbert-base-uncased-finetuned-sst-2-english
             "inputs": components.Textbox(label="Input"),
             "outputs": components.Label(label="Classification"),
             "preprocess": lambda x: {"inputs": x},
-            "postprocess": lambda r: {
-                i["label"].split(", ")[0]: i["score"] for i in r.json()[0]
-            },
+            "postprocess": lambda r: postprocess_label(
+                {i["label"].split(", ")[0]: i["score"] for i in r.json()[0]}
+            ),
         },
         "text-generation": {
+            # Example: gpt2
             "inputs": components.Textbox(label="Input"),
             "outputs": components.Textbox(label="Output"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: r.json()[0]["generated_text"],
         },
         "text2text-generation": {
+            # Example: valhalla/t5-small-qa-qg-hl
             "inputs": components.Textbox(label="Input"),
             "outputs": components.Textbox(label="Generated Text"),
             "preprocess": lambda x: {"inputs": x},
@@ -182,6 +253,7 @@ def get_models_interface(model_name, api_key, alias, **kwargs):
             "postprocess": lambda r: r.json()[0]["translation_text"],
         },
         "zero-shot-classification": {
+            # Example: facebook/bart-large-mnli
             "inputs": [
                 components.Textbox(label="Input"),
                 components.Textbox(label="Possible class names (" "comma-separated)"),
@@ -192,13 +264,15 @@ def get_models_interface(model_name, api_key, alias, **kwargs):
                 "inputs": i,
                 "parameters": {"candidate_labels": c, "multi_class": m},
             },
-            "postprocess": lambda r: {
-                r.json()["labels"][i]: r.json()["scores"][i]
-                for i in range(len(r.json()["labels"]))
-            },
+            "postprocess": lambda r: postprocess_label(
+                {
+                    r.json()["labels"][i]: r.json()["scores"][i]
+                    for i in range(len(r.json()["labels"]))
+                }
+            ),
         },
         "sentence-similarity": {
-            # example model: hf.co/sentence-transformers/distilbert-base-nli-stsb-mean-tokens
+            # Example: sentence-transformers/distilbert-base-nli-stsb-mean-tokens
             "inputs": [
                 components.Textbox(
                     value="That is a happy person", label="Source Sentence"
@@ -216,26 +290,26 @@ def get_models_interface(model_name, api_key, alias, **kwargs):
                     "sentences": [s for s in sentences.splitlines() if s != ""],
                 }
             },
-            "postprocess": lambda r: {
-                f"sentence {i}": v for i, v in enumerate(r.json())
-            },
+            "postprocess": lambda r: postprocess_label(
+                {f"sentence {i}": v for i, v in enumerate(r.json())}
+            ),
         },
         "text-to-speech": {
-            # example model: hf.co/julien-c/ljspeech_tts_train_tacotron2_raw_phn_tacotron_g2p_en_no_space_train
+            # Example: julien-c/ljspeech_tts_train_tacotron2_raw_phn_tacotron_g2p_en_no_space_train
             "inputs": components.Textbox(label="Input"),
             "outputs": components.Audio(label="Audio"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": encode_to_base64,
         },
         "text-to-image": {
-            # example model: hf.co/osanseviero/BigGAN-deep-128
+            # example model: osanseviero/BigGAN-deep-128
             "inputs": components.Textbox(label="Input"),
             "outputs": components.Image(label="Output"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": encode_to_base64,
         },
         "token-classification": {
-            # example model: hf.co/huggingface-course/bert-finetuned-ner
+            # example model: huggingface-course/bert-finetuned-ner
             "inputs": components.Textbox(label="Input"),
             "outputs": components.HighlightedText(label="Output"),
             "preprocess": lambda x: {"inputs": x},
@@ -243,6 +317,29 @@ def get_models_interface(model_name, api_key, alias, **kwargs):
         },
     }
 
+    if p in ["tabular-classification", "tabular-regression"]:
+        example_data = get_tabular_examples(model_name)
+        col_names, example_data = cols_to_rows(example_data)
+        example_data = [[example_data]] if example_data else None
+
+        pipelines[p] = {
+            "inputs": components.Dataframe(
+                label="Input Rows",
+                type="pandas",
+                headers=col_names,
+                col_count=(len(col_names), "fixed"),
+            ),
+            "outputs": components.Dataframe(
+                label="Predictions", type="array", headers=["prediction"]
+            ),
+            "preprocess": rows_to_cols,
+            "postprocess": lambda r: {
+                "headers": ["prediction"],
+                "data": [[pred] for pred in json.loads(r.text)],
+            },
+            "examples": example_data,
+        }
+
     if p is None or not (p in pipelines):
         raise ValueError("Unsupported pipeline type: {}".format(p))
 
@@ -258,10 +355,16 @@ def get_models_interface(model_name, api_key, alias, **kwargs):
             data = json.dumps(data)
         response = requests.request("POST", api_url, headers=headers, data=data)
         if not (response.status_code == 200):
+            errors_json = response.json()
+            errors, warns = "", ""
+            if errors_json.get("error"):
+                errors = f", Error: {errors_json.get('error')}"
+            if errors_json.get("warnings"):
+                warns = f", Warnings: {errors_json.get('warnings')}"
             raise ValueError(
-                "Could not complete request to HuggingFace API, Error {}".format(
-                    response.status_code
-                )
+                f"Could not complete request to HuggingFace API, Status Code: {response.status_code}"
+                + errors
+                + warns
             )
         if (
             p == "token-classification"
@@ -282,11 +385,12 @@ def get_models_interface(model_name, api_key, alias, **kwargs):
         "inputs": pipeline["inputs"],
         "outputs": pipeline["outputs"],
         "title": model_name,
+        "examples": pipeline.get("examples"),
     }
 
     kwargs = dict(interface_info, **kwargs)
+    kwargs["_api_mode"] = True  # So interface doesn't run pre/postprocess.
     interface = gradio.Interface(**kwargs)
-    interface.api_mode = True  # So interface doesn't run pre/postprocess.
     return interface
 
 
@@ -309,6 +413,40 @@ def get_spaces(model_name, api_key, alias, **kwargs):
         return get_spaces_blocks(model_name, config)
 
 
+async def get_pred_from_ws(
+    websocket: websockets.WebSocketClientProtocol, data: str, hash_data: str
+) -> Dict[str, Any]:
+    completed = False
+    while not completed:
+        msg = await websocket.recv()
+        resp = json.loads(msg)
+        if resp["msg"] == "queue_full":
+            raise exceptions.Error("Queue is full! Please try again.")
+        if resp["msg"] == "send_hash":
+            await websocket.send(hash_data)
+        elif resp["msg"] == "send_data":
+            await websocket.send(data)
+        completed = resp["msg"] == "process_completed"
+    return resp["output"]
+
+
+def get_ws_fn(ws_url):
+    async def ws_fn(data, hash_data):
+        async with websockets.connect(ws_url, open_timeout=10) as websocket:
+            return await get_pred_from_ws(websocket, data, hash_data)
+
+    return ws_fn
+
+
+def use_websocket(config, dependency):
+    queue_enabled = config.get("enable_queue", False)
+    queue_uses_websocket = version.parse(
+        config.get("version", "2.0")
+    ) >= version.Version("3.2")
+    dependency_uses_queue = dependency.get("queue", False) is not False
+    return queue_enabled and queue_uses_websocket and dependency_uses_queue
+
+
 def get_spaces_blocks(model_name, config):
     def streamline_config(config: dict) -> dict:
         """Streamlines the blocks config dictionary to fix components that don't render correctly."""
@@ -321,29 +459,45 @@ def get_spaces_blocks(model_name, config):
     config = streamline_config(config)
     api_url = "https://hf.space/embed/{}/api/predict/".format(model_name)
     headers = {"Content-Type": "application/json"}
+    ws_url = "wss://spaces.huggingface.tech/{}/queue/join".format(model_name)
+
+    ws_fn = get_ws_fn(ws_url)
 
     fns = []
     for d, dependency in enumerate(config["dependencies"]):
         if dependency["backend_fn"]:
 
-            def get_fn(outputs, fn_index):
+            def get_fn(outputs, fn_index, use_ws):
                 def fn(*data):
                     data = json.dumps({"data": data, "fn_index": fn_index})
-                    response = requests.post(api_url, headers=headers, data=data)
-                    result = json.loads(response.content.decode("utf-8"))
-                    try:
+                    hash_data = json.dumps(
+                        {"fn_index": fn_index, "session_hash": str(uuid.uuid4())}
+                    )
+                    if use_ws:
+                        result = utils.synchronize_async(ws_fn, data, hash_data)
                         output = result["data"]
-                    except KeyError:
-                        raise KeyError(
-                            f"Could not find 'data' key in response from external Space. Response received: {result}"
-                        )
+                    else:
+                        response = requests.post(api_url, headers=headers, data=data)
+                        result = json.loads(response.content.decode("utf-8"))
+                        try:
+                            output = result["data"]
+                        except KeyError:
+                            if "error" in result and "429" in result["error"]:
+                                raise TooManyRequestsError(
+                                    "Too many requests to the Hugging Face API"
+                                )
+                            raise KeyError(
+                                f"Could not find 'data' key in response from external Space. Response received: {result}"
+                            )
                     if len(outputs) == 1:
                         output = output[0]
                     return output
 
                 return fn
 
-            fn = get_fn(deepcopy(dependency["outputs"]), d)
+            fn = get_fn(
+                deepcopy(dependency["outputs"]), d, use_websocket(config, dependency)
+            )
             fns.append(fn)
         else:
             fns.append(None)
@@ -385,6 +539,8 @@ def get_spaces_interface(model_name, config, alias, **kwargs):
         try:
             output = result["data"]
         except KeyError:
+            if "error" in result and "429" in result["error"]:
+                raise TooManyRequestsError("Too many requests to the Hugging Face API")
             raise KeyError(
                 f"Could not find 'data' key in response from external Space. Response received: {result}"
             )
@@ -402,8 +558,8 @@ def get_spaces_interface(model_name, config, alias, **kwargs):
     config["fn"] = fn
 
     kwargs = dict(config, **kwargs)
+    kwargs["_api_mode"] = True
     interface = gradio.Interface(**kwargs)
-    interface.api_mode = True  # So interface doesn't run pre/postprocess.
     return interface
 
 
@@ -579,7 +735,6 @@ def load_from_pipeline(pipeline):
             data = pipeline(*data)
         else:
             data = pipeline(**data)
-        # print("Before postprocessing", data)
         output = pipeline_info["postprocess"](data)
         return output
 
