@@ -8,16 +8,17 @@ from __future__ import annotations
 import inspect
 import json
 import os
+import pkgutil
 import re
 import warnings
 import weakref
 from enum import Enum, auto
-from typing import TYPE_CHECKING, Any, Callable, List, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Callable, List, Optional
 
 from markdown_it import MarkdownIt
 from mdit_py_plugins.footnote import footnote_plugin
 
-from gradio import interpretation, utils
+from gradio import Examples, interpretation, utils
 from gradio.blocks import Blocks
 from gradio.components import (
     Button,
@@ -25,13 +26,11 @@ from gradio.components import (
     Interpretation,
     IOComponent,
     Markdown,
-    StatusTracker,
-    Variable,
+    State,
     get_component_instance,
 )
 from gradio.documentation import document, set_documentation_group
 from gradio.events import Changeable, Streamable
-from gradio.examples import Examples
 from gradio.external import load_from_pipeline  # type: ignore
 from gradio.flagging import CSVLogger, FlaggingCallback  # type: ignore
 from gradio.layouts import Column, Row, TabItem, Tabs
@@ -42,7 +41,7 @@ if TYPE_CHECKING:  # Only import for type checking (is False at runtime).
     import transformers
 
 
-@document("launch", "load", "from_pipeline", "integrate")
+@document("launch", "load", "from_pipeline", "integrate", "queue")
 class Interface(Blocks):
     """
     Interface is Gradio's main high-level class, and allows you to create a web-based GUI / demo
@@ -60,6 +59,7 @@ class Interface(Blocks):
         demo = gr.Interface(fn=image_classifier, inputs="image", outputs="label")
         demo.launch()
     Demos: hello_world, hello_world_3, gpt_j
+    Guides: quickstart, key_features, sharing_your_app, interface_state, reactive_interfaces, advanced_interface_features
     """
 
     # stores references to all currently existing Interface instances
@@ -149,11 +149,14 @@ class Interface(Blocks):
         flagging_dir: str = "flagged",
         flagging_callback: FlaggingCallback = CSVLogger(),
         analytics_enabled: Optional[bool] = None,
+        batch: bool = False,
+        max_batch_size: int = 4,
+        _api_mode: bool = False,
         **kwargs,
     ):
         """
         Parameters:
-            fn: the function to wrap an interface around. Often a machine learning model's prediction function.
+            fn: the function to wrap an interface around. Often a machine learning model's prediction function. Each parameter of the function corresponds to one input component, and the function should return a single value or a tuple of values, with each element in the tuple corresponding to one output component.
             inputs: a single Gradio component, or list of Gradio components. Components can either be passed as instantiated objects, or referred to by their string shortcuts. The number of input components should match the number of parameters in fn. If set to None, then only the output components will be displayed.
             outputs: a single Gradio component, or list of Gradio components. Components can either be passed as instantiated objects, or referred to by their string shortcuts. The number of output components should match the number of values returned by fn. If set to None, then only the input components will be displayed.
             examples: sample inputs for the function; if provided, appear below the UI components and can be clicked to populate the interface. Should be nested list, in which the outer list consists of samples and each inner list consists of an input corresponding to each input component. A string path to a directory of examples can also be provided. If there are multiple input components and a directory is provided, a log.csv file must be present in the directory to link corresponding inputs.
@@ -173,19 +176,18 @@ class Interface(Blocks):
             flagging_dir: what to name the directory where flagged data is stored.
             flagging_callback: An instance of a subclass of FlaggingCallback which will be called when a sample is flagged. By default logs to a local CSV file.
             analytics_enabled: Whether to allow basic telemetry. If None, will use GRADIO_ANALYTICS_ENABLED environment variable if defined, or default to True.
+            batch: If True, then the function should process a batch of inputs, meaning that it should accept a list of input values for each parameter. The lists should be of equal length (and be up to length `max_batch_size`). The function is then *required* to return a tuple of lists (even if there is only 1 output component), with each list in the tuple corresponding to one output component.
+            max_batch_size: Maximum number of inputs to batch together if this is called from the queue (only relevant if batch=True)
         """
         super().__init__(
             analytics_enabled=analytics_enabled,
             mode="interface",
             css=css,
             title=title,
+            theme=theme,
             **kwargs,
         )
 
-        if inspect.iscoroutinefunction(fn):
-            raise NotImplementedError(
-                "Async functions are not currently supported within interfaces. Please use Blocks API."
-            )
         self.interface_type = self.InterfaceTypes.STANDARD
         if (inputs is None or inputs == []) and (outputs is None or outputs == []):
             raise ValueError("Must provide at least one of `inputs` or `outputs`")
@@ -213,17 +215,30 @@ class Interface(Blocks):
         else:
             self.cache_examples = cache_examples or False
 
-        if "state" in inputs or "state" in outputs:
-            state_input_count = len([i for i in inputs if i == "state"])
-            state_output_count = len([o for o in outputs if o == "state"])
-            if state_input_count != 1 or state_output_count != 1:
-                raise ValueError(
-                    "If using 'state', there must be exactly one state input and one state output."
-                )
-            default = utils.get_default_args(fn)[inputs.index("state")]
-            state_variable = Variable(value=default)
-            inputs[inputs.index("state")] = state_variable
-            outputs[outputs.index("state")] = state_variable
+        state_input_indexes = [
+            idx for idx, i in enumerate(inputs) if i == "state" or isinstance(i, State)
+        ]
+        state_output_indexes = [
+            idx for idx, o in enumerate(outputs) if o == "state" or isinstance(o, State)
+        ]
+
+        if len(state_input_indexes) == 0 and len(state_output_indexes) == 0:
+            pass
+        elif len(state_input_indexes) != 1 or len(state_output_indexes) != 1:
+            raise ValueError(
+                "If using 'state', there must be exactly one state input and one state output."
+            )
+        else:
+            state_input_index = state_input_indexes[0]
+            state_output_index = state_output_indexes[0]
+            if inputs[state_input_index] == "state":
+                default = utils.get_default_args(fn)[state_input_index]
+                state_variable = State(value=default)
+            else:
+                state_variable = inputs[state_input_index]
+
+            inputs[state_input_index] = state_variable
+            outputs[state_output_index] = state_variable
 
             if cache_examples:
                 warnings.warn(
@@ -240,9 +255,7 @@ class Interface(Blocks):
         ]
 
         for component in self.input_components + self.output_components:
-            if not (
-                isinstance(component, IOComponent) or isinstance(component, Variable)
-            ):
+            if not (isinstance(component, IOComponent)):
                 raise ValueError(
                     f"{component} is not a valid input/output component for Interface."
                 )
@@ -274,10 +287,10 @@ class Interface(Blocks):
         else:
             raise ValueError("Invalid value for parameter: interpretation")
 
-        self.api_mode = False
+        self.api_mode = _api_mode
         self.fn = fn
         self.fn_durations = [0, 0]
-        self.__name__ = fn.__name__
+        self.__name__ = getattr(fn, "__name__", "fn")
         self.live = live
         self.title = title
 
@@ -287,14 +300,18 @@ class Interface(Blocks):
             cleantext = re.sub(CLEANER, "", raw_html)
             return cleantext
 
-        md = MarkdownIt(
-            "js-default",
-            {
-                "linkify": True,
-                "typographer": True,
-                "html": True,
-            },
-        ).use(footnote_plugin)
+        md = (
+            MarkdownIt(
+                "js-default",
+                {
+                    "linkify": True,
+                    "typographer": True,
+                    "html": True,
+                },
+            )
+            .use(footnote_plugin)
+            .enable("table")
+        )
 
         simple_description = None
         if description is not None:
@@ -368,19 +385,22 @@ class Interface(Blocks):
 
         self.favicon_path = None
 
-        data = {
-            "fn": fn,
-            "inputs": inputs,
-            "outputs": outputs,
-            "live": live,
-            "ip_address": self.ip_address,
-            "interpretation": interpretation,
-            "allow_flagging": allow_flagging,
-            "custom_css": self.css is not None,
-            "theme": self.theme,
-        }
-
         if self.analytics_enabled:
+            data = {
+                "mode": self.mode,
+                "fn": fn,
+                "inputs": inputs,
+                "outputs": outputs,
+                "live": live,
+                "ip_address": self.ip_address,
+                "interpretation": interpretation,
+                "allow_flagging": allow_flagging,
+                "custom_css": self.css is not None,
+                "theme": self.theme,
+                "version": pkgutil.get_data(__name__, "version.txt")
+                .decode("ascii")
+                .strip(),
+            }
             utils.initiated_analytics(data)
 
         utils.version_check()
@@ -437,11 +457,6 @@ class Interface(Blocks):
                 ]:
                     with Column(variant="panel"):
                         input_component_column = Column()
-                        if self.interface_type in [
-                            self.InterfaceTypes.INPUT_ONLY,
-                            self.InterfaceTypes.UNIFIED,
-                        ]:
-                            status_tracker = StatusTracker(cover_container=True)
                         with input_component_column:
                             for component in self.input_components:
                                 component.render()
@@ -451,7 +466,7 @@ class Interface(Blocks):
                             with interpret_component_column:
                                 for component in self.input_components:
                                     interpretation_set.append(Interpretation(component))
-                        with Row().style(mobile_collapse=False):
+                        with Row():
                             if self.interface_type in [
                                 self.InterfaceTypes.STANDARD,
                                 self.InterfaceTypes.INPUT_ONLY,
@@ -459,9 +474,19 @@ class Interface(Blocks):
                                 clear_btn = Button("Clear")
                                 if not self.live:
                                     submit_btn = Button("Submit", variant="primary")
+                                    # Stopping jobs only works if the queue is enabled
+                                    # We don't know if the queue is enabled when the interface
+                                    # is created. We use whether a generator function is provided
+                                    # as a proxy of whether the queue will be enabled.
+                                    # Using a generator function without the queue will raise an error.
+                                    if inspect.isgeneratorfunction(fn):
+                                        stop_btn = Button("Stop", variant="stop")
+
                             elif self.interface_type == self.InterfaceTypes.UNIFIED:
                                 clear_btn = Button("Clear")
                                 submit_btn = Button("Submit", variant="primary")
+                                if inspect.isgeneratorfunction(fn) and not self.live:
+                                    stop_btn = Button("Stop", variant="stop")
                                 if self.allow_flagging == "manual":
                                     flag_btns = render_flag_btns(self.flagging_options)
 
@@ -471,40 +496,48 @@ class Interface(Blocks):
                 ]:
 
                     with Column(variant="panel"):
-                        status_tracker = StatusTracker(cover_container=True)
                         for component in self.output_components:
-                            component.render()
-                        with Row().style(mobile_collapse=False):
+                            if not (isinstance(component, State)):
+                                component.render()
+                        with Row():
                             if self.interface_type == self.InterfaceTypes.OUTPUT_ONLY:
                                 clear_btn = Button("Clear")
                                 submit_btn = Button("Generate", variant="primary")
+                                if inspect.isgeneratorfunction(fn) and not self.live:
+                                    # Stopping jobs only works if the queue is enabled
+                                    # We don't know if the queue is enabled when the interface
+                                    # is created. We use whether a generator function is provided
+                                    # as a proxy of whether the queue will be enabled.
+                                    # Using a generator function without the queue will raise an error.
+                                    stop_btn = Button("Stop", variant="stop")
                             if self.allow_flagging == "manual":
                                 flag_btns = render_flag_btns(self.flagging_options)
                             if self.interpretation:
                                 interpretation_btn = Button("Interpret")
-            submit_fn = (
-                lambda *args: self.run_prediction(args)[0]
-                if len(self.output_components) == 1
-                else self.run_prediction(args)
-            )
             if self.live:
                 if self.interface_type == self.InterfaceTypes.OUTPUT_ONLY:
-                    super().load(submit_fn, None, self.output_components)
+                    super().load(self.fn, None, self.output_components)
                     submit_btn.click(
-                        submit_fn,
+                        self.fn,
                         None,
                         self.output_components,
                         api_name="predict",
-                        status_tracker=status_tracker,
+                        preprocess=not (self.api_mode),
+                        postprocess=not (self.api_mode),
+                        batch=batch,
+                        max_batch_size=max_batch_size,
                     )
                 else:
                     for component in self.input_components:
                         if isinstance(component, Streamable):
                             if component.streaming:
                                 component.stream(
-                                    submit_fn,
+                                    self.fn,
                                     self.input_components,
                                     self.output_components,
+                                    api_name="predict",
+                                    preprocess=not (self.api_mode),
+                                    postprocess=not (self.api_mode),
                                 )
                                 continue
                             else:
@@ -515,17 +548,33 @@ class Interface(Blocks):
                                 )
                         if isinstance(component, Changeable):
                             component.change(
-                                submit_fn, self.input_components, self.output_components
+                                self.fn,
+                                self.input_components,
+                                self.output_components,
+                                api_name="predict",
+                                preprocess=not (self.api_mode),
+                                postprocess=not (self.api_mode),
                             )
             else:
-                submit_btn.click(
-                    submit_fn,
+                pred = submit_btn.click(
+                    self.fn,
                     self.input_components,
                     self.output_components,
                     api_name="predict",
                     scroll_to_output=True,
-                    status_tracker=status_tracker,
+                    preprocess=not (self.api_mode),
+                    postprocess=not (self.api_mode),
+                    batch=batch,
+                    max_batch_size=max_batch_size,
                 )
+                if inspect.isgeneratorfunction(fn):
+                    stop_btn.click(
+                        None,
+                        inputs=None,
+                        outputs=None,
+                        cancels=[pred],
+                    )
+
             clear_btn.click(
                 None,
                 [],
@@ -546,16 +595,16 @@ class Interface(Blocks):
                 ),
                 _js=f"""() => {json.dumps(
                     [component.cleared_value if hasattr(component, "cleared_value") else None
-                    for component in self.input_components + self.output_components] + (
-                            [Column.update(visible=True)]
-                            if self.interface_type
-                            in [
-                                self.InterfaceTypes.STANDARD,
-                                self.InterfaceTypes.INPUT_ONLY,
-                                self.InterfaceTypes.UNIFIED,
-                            ]
-                            else []
-                        )
+                     for component in self.input_components + self.output_components] + (
+                        [Column.update(visible=True)]
+                        if self.interface_type
+                           in [
+                               self.InterfaceTypes.STANDARD,
+                               self.InterfaceTypes.INPUT_ONLY,
+                               self.InterfaceTypes.UNIFIED,
+                           ]
+                        else []
+                    )
                     + ([Column.update(visible=False)] if self.interpretation else [])
                 )}
                 """,
@@ -565,6 +614,7 @@ class Interface(Blocks):
                 def __init__(self, flagging_callback, flag_option=None):
                     self.flagging_callback = flagging_callback
                     self.flag_option = flag_option
+                    self.__name__ = "Flag"
 
                 def __call__(self, *flag_data):
                     self.flagging_callback.flag(flag_data, flag_option=self.flag_option)
@@ -585,16 +635,16 @@ class Interface(Blocks):
                             flag_method,
                             inputs=flag_components,
                             outputs=[],
-                            _preprocess=False,
+                            preprocess=False,
                             queue=False,
                         )
 
             if self.examples:
                 non_state_inputs = [
-                    c for c in self.input_components if not isinstance(c, Variable)
+                    c for c in self.input_components if not isinstance(c, State)
                 ]
                 non_state_outputs = [
-                    c for c in self.output_components if not isinstance(c, Variable)
+                    c for c in self.output_components if not isinstance(c, State)
                 ]
                 self.examples_handler = Examples(
                     examples=examples,
@@ -603,17 +653,17 @@ class Interface(Blocks):
                     fn=self.fn,
                     cache_examples=self.cache_examples,
                     examples_per_page=examples_per_page,
+                    _api_mode=_api_mode,
+                    batch=batch,
                 )
 
             if self.interpretation:
                 interpretation_btn.click(
-                    lambda *data: self.interpret(data)
-                    + [Column.update(visible=False), Column.update(visible=True)],
+                    self.interpret_func,
                     inputs=self.input_components + self.output_components,
                     outputs=interpretation_set
                     + [input_component_column, interpret_component_column],
-                    status_tracker=status_tracker,
-                    _preprocess=False,
+                    preprocess=False,
                 )
 
             if self.article:
@@ -621,15 +671,6 @@ class Interface(Blocks):
 
         self.config = self.get_config_file()
 
-    def __call__(self, *params):
-        if (
-            self.api_mode
-        ):  # skip the preprocessing/postprocessing if sending to a remote API
-            output = self.run_prediction(params, called_directly=True)
-        else:
-            output = self.process(params)
-        return output[0] if len(output) == 1 else output
-
     def __str__(self):
         return self.__repr__()
 
@@ -644,66 +685,17 @@ class Interface(Blocks):
             repr += "\n|-{}".format(str(component))
         return repr
 
-    def run_prediction(
-        self,
-        processed_input: List[Any],
-        called_directly: bool = False,
-    ) -> List[Any] | Tuple[List[Any], List[float]]:
-        """
-        Runs the prediction function with the given (already processed) inputs.
-        Parameters:
-            processed_input (list): A list of processed inputs.
-            called_directly (bool): Whether the prediction is being called directly (i.e. as a function, not through the GUI).
-        Returns:
-            predictions (list): A list of predictions (not post-processed).
-        """
-        if self.api_mode:  # Serialize the input
-            processed_input = [
-                input_component.serialize(processed_input[i], called_directly)
-                for i, input_component in enumerate(self.input_components)
-            ]
-
-        prediction = self.fn(*processed_input)
-
-        if prediction is None or len(self.output_components) == 1:
-            prediction = [prediction]
-
-        if self.api_mode:  # Deerialize the input
-            prediction = [
-                output_component.deserialize(prediction[i])
-                for i, output_component in enumerate(self.output_components)
-            ]
-
-        return prediction
-
-    def process(self, raw_input: List[Any]) -> Tuple[List[Any], List[float]]:
-        """
-        First preprocesses the input, then runs prediction using
-        self.run_prediction(), then postprocesses the output.
-        Parameters:
-            raw_input: a list of raw inputs to process and apply the prediction(s) on.
-        Returns:
-            processed output: a list of processed  outputs to return as the prediction(s).
-            duration: a list of time deltas measuring inference time for each prediction fn.
-        """
-        processed_input = [
-            input_component.preprocess(raw_input[i])
-            for i, input_component in enumerate(self.input_components)
+    async def interpret_func(self, *args):
+        return await self.interpret(args) + [
+            Column.update(visible=False),
+            Column.update(visible=True),
         ]
-        predictions = self.run_prediction(processed_input)
-        processed_output = [
-            output_component.postprocess(predictions[i])
-            if predictions[i] is not None
-            else None
-            for i, output_component in enumerate(self.output_components)
-        ]
-        return processed_output
 
-    def interpret(self, raw_input: List[Any]) -> List[Any]:
+    async def interpret(self, raw_input: List[Any]) -> List[Any]:
         return [
             {"original": raw_value, "interpretation": interpretation}
             for interpretation, raw_value in zip(
-                interpretation.run_interpret(self, raw_input)[0], raw_input
+                (await interpretation.run_interpret(self, raw_input))[0], raw_input
             )
         ]
 
@@ -721,7 +713,7 @@ class Interface(Blocks):
             else:
                 raw_input.append(input_component.test_input)
         else:
-            self.process(raw_input)
+            self(raw_input)
             print("PASSED")
 
 
@@ -730,22 +722,35 @@ class TabbedInterface(Blocks):
     """
     A TabbedInterface is created by providing a list of Interfaces, each of which gets
     rendered in a separate tab.
-    Demos: sst_or_tts
+    Demos: stt_or_tts
     """
 
     def __init__(
-        self, interface_list: List[Interface], tab_names: Optional[List[str]] = None
+        self,
+        interface_list: List[Interface],
+        tab_names: Optional[List[str]] = None,
+        theme: str = "default",
+        analytics_enabled: Optional[bool] = None,
+        css: Optional[str] = None,
     ):
         """
         Parameters:
             interface_list: a list of interfaces to be rendered in tabs.
             tab_names: a list of tab names. If None, the tab names will be "Tab 1", "Tab 2", etc.
+            theme: which theme to use - right now, only "default" is supported.
+            analytics_enabled: whether to allow basic telemetry. If None, will use GRADIO_ANALYTICS_ENABLED environment variable or default to True.
+            css: custom css or path to custom css file to apply to entire Blocks
         Returns:
             a Gradio Tabbed Interface for the given interfaces
         """
+        super().__init__(
+            theme=theme,
+            analytics_enabled=analytics_enabled,
+            mode="tabbed_interface",
+            css=css,
+        )
         if tab_names is None:
             tab_names = ["Tab {}".format(i) for i in range(len(interface_list))]
-        super().__init__()
         with self:
             with Tabs():
                 for (interface, tab_name) in zip(interface_list, tab_names):
