@@ -7,4 +7,4 @@ function make_script(src) {
     script.src = src;
     document.head.appendChild(script);
 }
-make_script("https://gradio.s3-us-west-2.amazonaws.com/3.8/assets/index.7611b64a.js");
+make_script("https://gradio.s3-us-west-2.amazonaws.com/3.8.2/assets/index.499bf81d.js");
