@@ -9,6 +9,7 @@ import gradio as gr
 os.environ["GRADIO_ANALYTICS_ENABLED"] = "False"
 
 
+@patch("gradio.examples.CACHED_FOLDER", tempfile.mkdtemp())
 class TestExamples:
     def test_handle_single_input(self):
         examples = gr.Examples(["hello", "hi"], gr.Textbox())
@@ -42,7 +43,47 @@ class TestExamples:
             [gr.media_data.BASE64_IMAGE, "hi"],
         ]
 
+    @pytest.mark.asyncio
+    async def test_no_preprocessing(self):
+        with gr.Blocks():
+            image = gr.Image()
+            textbox = gr.Textbox()
 
+            examples = gr.Examples(
+                examples=["test/test_files/bus.png"],
+                inputs=image,
+                outputs=textbox,
+                fn=lambda x: x,
+                cache_examples=True,
+                preprocess=False,
+            )
+
+        prediction = await examples.load_from_cache(0)
+        assert prediction == [gr.media_data.BASE64_IMAGE]
+
+    @pytest.mark.asyncio
+    async def test_no_postprocessing(self):
+        def im(x):
+            return [gr.media_data.BASE64_IMAGE]
+
+        with gr.Blocks():
+            text = gr.Textbox()
+            gall = gr.Gallery()
+
+            examples = gr.Examples(
+                examples=["hi"],
+                inputs=text,
+                outputs=gall,
+                fn=im,
+                cache_examples=True,
+                postprocess=False,
+            )
+
+        prediction = await examples.load_from_cache(0)
+        assert prediction[0][0]["data"] == gr.media_data.BASE64_IMAGE
+
+
+@patch("gradio.examples.CACHED_FOLDER", tempfile.mkdtemp())
 class TestExamplesDataset:
     def test_no_headers(self):
         examples = gr.Examples("test/test_files/images_log", [gr.Image(), gr.Text()])
@@ -64,21 +105,6 @@ class TestExamplesDataset:
 
 @patch("gradio.examples.CACHED_FOLDER", tempfile.mkdtemp())
 class TestProcessExamples:
-    @pytest.mark.asyncio
-    async def test_predict_example(self):
-        io = gr.Interface(lambda x: "Hello " + x, "text", "text", examples=[["World"]])
-        prediction = await io.examples_handler.predict_example(0)
-        assert prediction[0] == "Hello World"
-
-    @pytest.mark.asyncio
-    async def test_coroutine_process_example(self):
-        async def coroutine(x):
-            return "Hello " + x
-
-        io = gr.Interface(coroutine, "text", "text", examples=[["World"]])
-        prediction = await io.examples_handler.predict_example(0)
-        assert prediction[0] == "Hello World"
-
     @pytest.mark.asyncio
     async def test_caching(self):
         io = gr.Interface(
@@ -86,11 +112,35 @@ class TestProcessExamples:
             "text",
             "text",
             examples=[["World"], ["Dunya"], ["Monde"]],
+            cache_examples=True,
         )
-        await io.examples_handler.cache_interface_examples()
         prediction = await io.examples_handler.load_from_cache(1)
         assert prediction[0] == "Hello Dunya"
 
+    @pytest.mark.asyncio
+    async def test_caching_image(self):
+        io = gr.Interface(
+            lambda x: x,
+            "image",
+            "image",
+            examples=[["test/test_files/bus.png"]],
+            cache_examples=True,
+        )
+        prediction = await io.examples_handler.load_from_cache(0)
+        assert prediction[0].startswith("data:image/png;base64,iVBORw0KGgoAAA")
+
+    @pytest.mark.asyncio
+    async def test_caching_audio(self):
+        io = gr.Interface(
+            lambda x: x,
+            "audio",
+            "audio",
+            examples=[["test/test_files/audio_sample.wav"]],
+            cache_examples=True,
+        )
+        prediction = await io.examples_handler.load_from_cache(0)
+        assert prediction[0]["data"].startswith("data:audio/wav;base64,UklGRgA/")
+
     @pytest.mark.asyncio
     async def test_caching_with_update(self):
         io = gr.Interface(
@@ -98,8 +148,8 @@ class TestProcessExamples:
             "text",
             "image",
             examples=[["World"], ["Dunya"], ["Monde"]],
+            cache_examples=True,
         )
-        await io.examples_handler.cache_interface_examples()
         prediction = await io.examples_handler.load_from_cache(1)
         assert prediction[0] == {"visible": False, "__type__": "update"}
 
@@ -110,8 +160,8 @@ class TestProcessExamples:
             "text",
             ["text", "image"],
             examples=[["World"], ["Dunya"], ["Monde"]],
+            cache_examples=True,
         )
-        await io.examples_handler.cache_interface_examples()
         prediction = await io.examples_handler.load_from_cache(1)
         assert prediction[0] == {"lines": 4, "value": "hello", "__type__": "update"}
 
@@ -127,7 +177,6 @@ class TestProcessExamples:
             examples=["abc"],
             cache_examples=True,
         )
-        await io.examples_handler.cache_interface_examples()
         prediction = await io.examples_handler.load_from_cache(0)
         assert prediction == [{"lines": 4, "__type__": "update"}, {"label": "lion"}]
 
@@ -187,3 +236,43 @@ class TestProcessExamples:
                     examples=[["foo", None, None], ["bar", 2, 3]],
                     cache_examples=True,
                 )
+
+    @pytest.mark.asyncio
+    async def test_caching_with_batch(self):
+        def trim_words(words, lens):
+            trimmed_words = []
+            for w, l in zip(words, lens):
+                trimmed_words.append(w[:l])
+            return [trimmed_words]
+
+        io = gr.Interface(
+            trim_words,
+            ["textbox", gr.Number(precision=0)],
+            ["textbox"],
+            batch=True,
+            max_batch_size=16,
+            examples=[["hello", 3], ["hi", 4]],
+            cache_examples=True,
+        )
+        prediction = await io.examples_handler.load_from_cache(0)
+        assert prediction == ["hel"]
+
+    @pytest.mark.asyncio
+    async def test_caching_with_batch_multiple_outputs(self):
+        def trim_words(words, lens):
+            trimmed_words = []
+            for w, l in zip(words, lens):
+                trimmed_words.append(w[:l])
+            return trimmed_words, lens
+
+        io = gr.Interface(
+            trim_words,
+            ["textbox", gr.Number(precision=0)],
+            ["textbox", gr.Number(precision=0)],
+            batch=True,
+            max_batch_size=16,
+            examples=[["hello", 3], ["hi", 4]],
+            cache_examples=True,
+        )
+        prediction = await io.examples_handler.load_from_cache(0)
+        assert prediction == ["hel", "3"]
